LOOK_BACK_DAYS = 30
MONTHS = 12
DEFAULT_ROUND_DIGITS = 3
PUBLIC_ACCOUNTS_COST_REPORTS = 'Accounts'
HOURS_IN_SECONDS = 3600
HOURS_IN_DAY = 24
HOURS_IN_MONTH = 720
TOTAL_BYTES_IN_KIB = 1024

DATE_FORMAT = "%Y-%m-%d"
DATE_TIME_FORMAT_T = "%Y-%m-%dT%h:%m"
UNUSED_DAYS = 7

# Policy Values
# Instance Idle
INSTANCE_IDLE_DAYS = 7
INSTANCE_IDLE_CPU_PERCENTAGE = 2
INSTANCE_IDLE_NETWORK_IN_KILO_BYTES = 5  # In KiB
INSTANCE_IDLE_NETWORK_OUT_KILO_BYTES = 5  # In KiB
EC2_NAMESPACE = 'AWS/EC2'
CLOUDWATCH_METRICS_AVAILABLE_DAYS = 14
AWS_DEFAULT_GLOBAL_REGION = 'us-east-1'
